/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package org.bsf.listOfValues.util;

import org.apache.commons.lang.StringUtils;
import org.bsf.listOfValues.LovMetaDataItem;
import org.bsf.listOfValues.lovValue.DefaultLovValue;
import org.bsf.listOfValues.lovValue.LovValue;

import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * This utility class provides some utility methods that can be applied to
 * the LOV (transformation of lov in a string and reverse...).
 *
 * @see LovValue
 * @see org.bsf.listOfValues.LovServiceBean
 */
public class LovUtil {
    private static final String DATA_SEPARATOR = "\t";
    private static final String LINE_SEPARATOR = "\n";
    private static final String TOKEN_TO_REPLACE = "#";

    /**
     * Used to transform a Lov into a String.
     *
     * @param p_lov The LOV per itself.
     * @param p_metaDatas The metadata for the given LOV.
     *
     * @throws IllegalArgumentException if any parameter is null.
     *
     * @see #stringToSortedLov
     */
    public static String lovToString( List p_lov, List p_metaDatas ) {
        if ( p_lov == null || p_metaDatas == null ) {
            String msg = "Need non null LOV and METADATA to make a String of the LOV...";

            throw new IllegalArgumentException( msg );
        }

        StringBuffer buffer = new StringBuffer();

        // We put the metadata on the first line of the StringBuffer
        Iterator metaDataIterator = p_metaDatas.iterator();

        while ( metaDataIterator.hasNext() ) {
            LovMetaDataItem item = (LovMetaDataItem) metaDataIterator.next();

            buffer.append( item.getFieldName() );
            buffer.append( DATA_SEPARATOR );
            buffer.append( item.getFieldType() );
            buffer.append( DATA_SEPARATOR );
        }

        buffer.append( LINE_SEPARATOR );

        // And then add a LovValue per line
        metaDataIterator = p_lov.iterator();

        while ( metaDataIterator.hasNext() ) {
            LovValue lovValue = (LovValue) metaDataIterator.next();

            buffer.append( lovValue.getOid() );
            buffer.append( DATA_SEPARATOR );

            for ( int i = 0 ; i < p_metaDatas.size() ; i++ ) {
                buffer.append( lovValue.getFieldValue( ( (LovMetaDataItem) p_metaDatas.get( i ) ).getFieldName() ) );
                buffer.append( DATA_SEPARATOR );
            }

            buffer.append( LINE_SEPARATOR );
        }

        return buffer.toString();
    }

    /**
     * Makes a LOV of a given String.
     *
     * @param p_stringLov The String to build the LOV from.
     *
     * @return The LOV corresponding to the given String.
     *
     * @throws IllegalArgumentException if the given String is null.
     *
     * @see #lovToString
     */
    public static List stringToSortedLov( String p_stringLov ) {
        if ( p_stringLov == null ) {
            throw new IllegalArgumentException( "Need a non null String to restore a LOV..." );
        }

        List result = new Vector();
        Vector metaDatas = new Vector();

        String firstLine = null;
        StringTokenizer lines = new StringTokenizer( p_stringLov, LINE_SEPARATOR );

        // If the string starts with a line separator, it means that we just have a OID
        if ( !p_stringLov.startsWith( LINE_SEPARATOR ) && lines.hasMoreTokens() ) {
            firstLine = lines.nextToken();

            // We take care of the metadata
            if ( firstLine != null ) {
                StringTokenizer mDataItems = new StringTokenizer( firstLine, DATA_SEPARATOR );
                String fieldName = null;
                String fieldType = null;

                while ( mDataItems.hasMoreTokens() ) {
                    //We always have a field fieldName and a field fieldType
                    fieldName = mDataItems.nextToken();
                    fieldType = mDataItems.nextToken();

                    LovMetaDataItem metaDataItem = new LovMetaDataItem( fieldName, fieldType );
                    metaDatas.add( metaDataItem );
                }
            }
        }

        // And now let's take care of the data themselves
        String line = null;
        StringTokenizer field = null;
        DefaultLovValue lovValue = null;
        String strField = null;
        LovMetaDataItem metaDataItem = null;
        Object lovObject = null;

        int i = 0;

        // For each remaining line
        while ( lines.hasMoreTokens() ) {
            line = lines.nextToken();
            field = new StringTokenizer( line, DATA_SEPARATOR );

            // We handle the OID
            if ( field.hasMoreTokens() ) { //always true
                lovValue = new DefaultLovValue( new Long( field.nextToken() ) );
            }

            // We handle each field
            while ( field.hasMoreTokens() ) {
                strField = field.nextToken();

                metaDataItem = (LovMetaDataItem) metaDatas.get( i );
                String fieldType = metaDataItem.getFieldType();
                String fieldName = metaDataItem.getFieldName();

                if ( strField.equals( "null" ) ) {
                    lovObject = null;
                } else {
                    try {
                        // We handle the found object with its defined type
                        if ( fieldType.equals( LovTypeManager.STRING_TYPE ) ) lovObject = strField;
                        if ( fieldType.equals( LovTypeManager.BOOLEAN_TYPE ) ) lovObject = new Boolean( strField );
                        if ( fieldType.equals( LovTypeManager.FLOAT_TYPE ) ) lovObject = new Float( strField );
                        if ( fieldType.equals( LovTypeManager.DOUBLE_TYPE ) ) lovObject = new Double( strField );
                        if ( fieldType.equals( LovTypeManager.INTEGER_TYPE ) ) lovObject = new Integer( strField );
                        if ( fieldType.equals( LovTypeManager.LONG_TYPE ) ) lovObject = new Long( strField );
                    } catch( Exception ex ) {
                        // Too bad we can't handle it... It will remain null...
                        lovObject = null;
                    }
                }

                // We add the field to the LovValue
                lovValue.addField( fieldName, lovObject );

                // Let's handle the next field
                i++;
            }

            // Back to the first field
            i = 0;

            // And we add the created LovValue to the LOV (ie. the List)
            result.add( lovValue );
        }

        return result;
    }

    /**
     * Replaces, in the given source string, the '#' token by the given string
     * to use. The TOKEN_TO_REPLACE should only found once in each source string
     * (you have to pay attention, all tokens will be replaced by the same
     * string...).
     *
     * @param p_sourceString The String to use as the source.
     * @param p_stringToUse The String to use as replacement for the '#' token(s)
     *
     * @return A string taken from the source string with the given replacment
     * string in place of the '#' token(s). Null if the source string is null.
     */
    public static String replaceToken( String p_sourceString, String p_stringToUse ) {
        if ( p_sourceString == null ) {
            return p_sourceString;
        } else {
            return StringUtils.replace( p_sourceString, TOKEN_TO_REPLACE, p_stringToUse );
        }
    }
}